class settings:
	AUTH_LDAP_SERVER_URI='ldap://10.140.0.5'
	AUTH_LDAP_BIND_DN='cn=admin,dc=c,dc=majestic-lead-196504,dc=internal'
	AUTH_LDAP_BIND_PASSWORD='admin'
	base_dn='dc=c,dc=majestic-lead-196504,dc=internal'
        log_path = 'ldap.log'
	def __init__(self,uri=None,binddn=None,password=None,basedn=None,log_path=None):
		if(uri!=None):
			self.AUTH_LDAP_SERVER_URI=uri
		if(binddn!=None):
			self.AUTH_LDAP_BIND_DN=binddn
		if(password!=None):
			self.AUTH_LDAP_BIND_PASSWORD=password
		if(basedn!=None):
			self.base_dn=basedn
		if(log_path != None):
			self.log_path=log_path



# -*- coding:utf-8 -*-
import ldap
import ldap.modlist as modlist

import pdb
import logging
#logger = logging.getLogger()
import sys
reload(sys)
sys.setdefaultencoding('utf8')


from ldapAdmin import settings
LDAP_URI = settings.AUTH_LDAP_SERVER_URI
LDAP_USER = settings.AUTH_LDAP_BIND_DN
LDAP_PASS = settings.AUTH_LDAP_BIND_PASSWORD
BASE_DN = settings.base_dn
LOG_PATH = settings.log_path
logging.basicConfig(filename=LOG_PATH,level=logging.ERROR)

class LDAPTool(object):
    def __init__(self,
                 ldap_uri=None,
                 base_dn=None,
                 user=None,
                 password=None):
       
#        initialize
#        :param ldap_uri: ldap_uri
#        :param base_dn:base dn
#        :param user: default user
#        :param password: default password
#        :return:
        
        if not ldap_uri:
            ldap_uri = LDAP_URI
        if not base_dn:
            self.base_dn = BASE_DN
        if not user:
            self.admin_user = LDAP_USER
        if not password:
            self.admin_password = LDAP_PASS
        try:
            self.ldapconn = ldap.initialize(ldap_uri)  # open() for older versions
            self.ldapconn.simple_bind(self.admin_user, self.admin_password)
        except ldap.LDAPError, e:
            logging.error('ldap conn failed，because: %s' % str(e))

    def ldap_search_dn(self, value=None, value_type='uid'):
        """
        :param value: uid or group cn
        :param value_type: user uid|cn
        :return: search result,format is [(dn1,{user1 info}),(dn1,{user1 info}),(dn1,{user1 info}) ... ]
        """
        obj = self.ldapconn
        obj.protocal_version = ldap.VERSION3
        searchScope = ldap.SCOPE_SUBTREE
        retrieveAttributes = None
        if value_type == 'cn':
            searchFilter = "cn=" + value
        else:
            searchFilter = "uid=" + value
        try:
            ldap_result_id = obj.search(
                base=self.base_dn,
                scope=searchScope,
                filterstr=searchFilter,
                attrlist=retrieveAttributes
            )
            result_type, result_data = obj.result(ldap_result_id, 0)
            if result_type == ldap.RES_SEARCH_ENTRY:
                return result_data
            else:
                return None
        except ldap.LDAPError, e:
            logging.error('ldap search %s failed, because: %s' % (value, str(e)))

    def ldap_get_user(self, uid=None):
        """
        :param uid:
        :return: {‘uid’:'zhangsan','mail':'zhangsan@xxx.com','cn':'张三'}
        """
        result = None
        try:
            search = self.ldap_search_dn(value=uid, value_type=uid)
            if search is None:
                raise ldap.LDAPError('corresponding id not found')
            for user in search:
                if user[1]['uid'][0] == uid:
#                    pdb.set_trace()
		    result = {
                        'uid': uid,
			'uidNumber': int(user[1]['uidNumber'][0]),
			'gidNumber': int(user[1]['gidNumber'][0]),
                        #'mail': user[1]['mail'][0] if  user[1]['mail'] !=None else '',
                        'cn': user[1]['cn'][0],
                    }
		    if(user[1].has_key('mail')):
			result['mail'] = user[1]['mail'][0]
		    
        except Exception, e:
            logging.error('get user %s failed，because: %s' % (uid, str(e)))
        return result

    def __ldap_getgid(self, cn="kevindwliu"):
        """
        get cn gid
        :param cn: group cn
        :return: cn gidNumber
        """
        obj = self.ldapconn
        obj.protocal_version = ldap.VERSION3
        searchScope = ldap.SCOPE_SUBTREE
        retrieveAttributes = None
        searchFilter = "cn=" + cn
        try:
            ldap_result_id = obj.search(
                base="ou=groups,%s" % self.base_dn,
                scope=searchScope,
                filterstr=searchFilter,
                attrlist=retrieveAttributes
            )
            result_type, result_data = obj.result(ldap_result_id, 0)
            if result_type == ldap.RES_SEARCH_ENTRY:
                return result_data[0][1].get('gidNumber')[0]
            else:
                return None
        except ldap.LDAPError, e:
            logging.error('get gid failed，because: %s' % str(e))

    def __get_max_uidNumber(self):
        """ 
        :param: None
        :return: max uidNumber
        """
        f = None
	try:
	    f = open('max_uid.txt','r')
	    last_max = int(f.read())
	    return last_max
	except IOError,e:
	    print e
	    	    
	finally:
	    if f:
		f.close()


        obj = self.ldapconn
        obj.protocal_version = ldap.VERSION3
        searchScope = ldap.SCOPE_SUBTREE
        retrieveAttributes = ['uidNumber']
        searchFilter = "uid=*"
	#pdb.set_trace()
        try:
            ldap_result = obj.search(
                base="ou=people,%s" % self.base_dn,
                scope=searchScope,
                filterstr=searchFilter,
                attrlist=retrieveAttributes
            )
            #result_set = [i]
            result = 0
	    while True:
                result_type, result_data = obj.result(ldap_result, 0)
                if not result_data:
                    break
                else:
                    if result_type == ldap.RES_SEARCH_ENTRY:
                        tmpuid = int(result_data[0][1].get('uidNumber')[0])
			if(tmpuid>result):		    
	    		    result = tmpuid

            f = None 
	    try:
		f = open('max_uid.txt','w')
		f.write(str(result))
	    except Exception,e:
		print e
	    finally:
		if f:
		    f.close()

 	    return result
        except ldap.LDAPError, e:
            logging.error('get max uid failed，because: %s' % str(e))

    def ldap_add_user(self, cn, mail, username, password):
        """
        add ldap user
        :param cn, mail, username, password
        :return: True/None
        """
        result = None
        try:
            #pdb.set_trace()
	    obj = self.ldapconn
            obj.protocal_version = ldap.VERSION3

            addDN = "uid=%s,ou=people,%s" % (username, BASE_DN)
            attrs = {}
            attrs['objectclass'] = ['top', 'person', 'inetOrgPerson', 'posixAccount', 'organizationalPerson']
            attrs['cn'] = str(cn)
            attrs['homeDirectory'] = str('/home/%s' % username)
            attrs['loginShell'] = '/bin/bash'
            attrs['mail'] = str(mail)
            attrs['sn'] = str(username)
            attrs['uid'] = str(username)
            attrs['userPassword'] = str(password)
            attrs['uidNumber'] = str(self.__get_max_uidNumber()+1)
            attrs['gidNumber'] = str(1001)#self.__ldap_getgid(cn='kevindwliu')
            ldif = ldap.modlist.addModlist(attrs)
#           pdb.set_trace()
	    obj.add_s(addDN, ldif)
            obj.unbind_s()
            result = True
	    
	    print('add user success!')
            f = None
            try:
		f = open('max_uid.txt','w')
		f.write(attrs['uidNumber'])
	    except IOError,e:
		print e
	    finally:
		if f:
		    f.close()
	except ldap.LDAPError, e:
            print('add user failed:%s'% str(e))
	    logging.error("add user %s failed，because: %s" % (username, str(e)))
        return result

    def check_user_belong_to_group(self, uid, group_cn='kevindwliu'):
        """
        :param uid: uid , Ex: 'admin'
        :param group_cn:cn , Ex: ''
        :return: True|None
        """
        result = None
        try:
            search = self.ldap_search_dn(value=group_cn, value_type='cn')
            if search is None:
                raise ldap.LDAPError('no result for id')

            member_list = search[0][1].get('memberUid', [])
            if uid in member_list:
                result = True
        except ldap.LDAPError, e:
            logging.error('get user %s and %s failed，because: %s' % (uid, group_cn, str(e)))
        return result

    def check_user_status(self, uid):
        """
        :param uid: uid
        :return: 200: user ok
                 404: user does not exists
                 403: user forbidden
        """
        result = 404
        try:
            target_cn = self.ldap_get_user(uid=uid)
            if target_cn is None: 
                result = 404
                logging.debug("%s uid no result" % uid)
            else:
                if self.check_user_belong_to_group(uid=uid, group_cn='forbidden'):
                    result = 403
                else:
                    result = 200
        except ldap.LDAPError, e:
            logging.error("check user %s status failed,because %s" % (uid, str(e)))
        return result

    def ldap_update_password(self, uid, new_password):
        """
        
        :param uid: uid，password
        :return: True|None
        """
        result = None
        try:
            obj = self.ldapconn
            obj.protocal_version = ldap.VERSION3
            modifyDN = "uid=%s,ou=People,%s" % (uid, BASE_DN)
            obj.modify_s(modifyDN, [(ldap.MOD_REPLACE, 'userPassword', [str(new_password)])])
            obj.unbind_s()
            result = True
        except ldap.LDAPError, e:
            logging.error("%s reset password failed，because: %s" % (uid, str(e)))
        return result



import ldap
import os
import json



def getHomeDir(userName):
	home_dir = os.path.join('/home',userName)
	walked = False
	for dir,subdirs,files in os.walk(home_dir):
		walked = True
		print('Found dir: %s' % dir)
		d = {'name':dir}	
		for subdir in subdirs:
			print ('\td %s'% subdir)
		for file in files:
			print ('\tf %s'% file)	
	if(not walked):
		print('get %s directory failed: permission denied' % userName)
		return None
	else:
		print('%s home directory walk finished' % userName)
		return path_to_dict(home_dir)



def path_to_dict(path):
    d = {'name': os.path.basename(path)}
    if os.path.isdir(path):
        d['type'] = "directory"
        d['children'] = [path_to_dict(os.path.join(path,x)) for x in os.listdir(path)]
    else:
        d['type'] = "file"
    return d







import ldap
import pdb
import printDir
import os
import ldapTool2
import logging
logging.basicConfig(filename='ldap.log',level=logging.ERROR)
from ldapAdmin import settings
ldappath = settings.AUTH_LDAP_SERVER_URI
ldapadmin = settings.AUTH_LDAP_BIND_DN
ldappasswd = settings.AUTH_LDAP_BIND_PASSWORD
baseDn = settings.base_dn

#ldappath = 'ldap://10.140.0.5'
#baseDn = 'dc=c,dc=majestic-lead-196504,dc=internal'
#ldapadmin = 'admin'
#ldappasswd = 'admin'


def LDAPlogin(userName,password):
	try:
		if(password == ''):
			print "password is empty"
			return
		
		dn = getDn(userName)
		if(dn == ''):
			raise Exception('user does not exits')
			#print "user does not exits"
			#return 
		ldap_server = ldap.initialize(ldappath)
		print ldap_server.simple_bind_s(dn,password)
		print "%s login Ok" % userName
#		flag = raw_input("get %s 's home directory? y/n: " % userName)
#		if(flag=='y'):
#			jsonResult = printDir.getHomeDir(userName)
#			print JSON.stringify(jsonResult,null,' ')
#			#print ('cannot access!' if dir==None else dir)
	except Exception,e:
		print "%s login failed: %s" % (userName,e)
		logging.error("%s login failed: %s" % (userName,e))
		return
	
	if(not os.path.exists('/home/%s' % userName)):
		try:
			tool = ldapTool2.LDAPTool()
			result = tool.ldap_get_user(userName)
			os.makedirs('/home/%s' % userName)
			os.chown('/home/%s' % userName,result['uidNumber'],result['gidNumber'])
		except Exception,e:
			print('make home dir failed,%s'% str(e))
			logging.error('make home dir failed,%s'% str(e))

	jsonResult = printDir.getHomeDir(userName)
	if(jsonResult is None):
		print("get json failed")
		return
	flag = raw_input("get %s 's home directory in json? y/n: " % userName)
        if(flag=='y'):
                print jsonResult
                #print ('cannot access!' if dir==None else dir)




def getDn(userName,trynum=3):
	i = 0
	isFound = 0
	foundResult = ''
	print('getting Dn...')
	#pdb.set_trace()
	while(i<trynum):
		isfound,foundResult = _validateLDAPUser(userName)
		if(isfound):
			break
		i+=1
	#print('Dn is %s' % foundResult)
	return foundResult	


def _validateLDAPUser(user): #return isfound and foundresult
	print('validating user')
	try:
		l = ldap.initialize(ldappath)
		l.protocol_version = ldap.VERSION3
		l.simple_bind(ldapadmin,ldappasswd)
		
		searchScope = ldap.SCOPE_SUBTREE
		searchFiltername = 'uid'
		retrieveAttributes = None
		searchFilter = '(' + searchFiltername + '=' + user + ')'
		#pdb.set_trace()		
		ldap_result_id = l.search(baseDn,searchScope,searchFilter,retrieveAttributes)
		result_type,result_data = l.result(ldap_result_id,1)
		if(len(result_data) != 0):
			r_a,r_b = result_data[0]
	        	return 1, r_a
		else:
			return 0,''
	
	except ldap.LDAPError,e:
		print e
		logging.error(str(e))
		return 0,''
	
	finally:
		l.unbind()
		del l



from setuptools import setup, find_packages 
setup(
    name = 'nebulaLogin',
    version = '0.4',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
      ],
    license = 'MIT License',
    install_requires = ['python-ldap'],
 
    author = 'kevindwliu',
    author_email = 'kevindwliu@outlook.com',
    
    packages = find_packages(),
    platforms = 'any',
)
